// Generated by CoffeeScript 1.12.7
(function() {
  var RedisLocker, Redlock,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  Redlock = require('redlock');

  RedisLocker = (function() {
    function RedisLocker(arg) {
      this.redis = arg.redis, this.uuid = arg.uuid, this.timestamp = arg.timestamp;
      this._isFutureTimestamp = bind(this._isFutureTimestamp, this);
      this.lockBlock = bind(this.lockBlock, this);
      this.redlock = new Redlock([this.redis], {
        retryCount: 60,
        retryDelay: 500
      });
      if (this.timestamp == null) {
        throw new Error('NO TIME');
      }
      if (this.uuid == null) {
        throw new Error('NO UUID');
      }
    }

    RedisLocker.prototype.lockBlock = function(callback) {
      return this.redlock.lock("lock:uuid:" + this.uuid, 30000, (function(_this) {
        return function(error, lock) {
          var unlockCallback;
          if (error != null) {
            return callback(error);
          }
          if (lock == null) {
            return _.defer(_this.lockBlock, callback);
          }
          unlockCallback = function(error) {
            return lock.unlock(function() {
              return callback(error);
            });
          };
          return _this._isFutureTimestamp(function(error, canProcess) {
            if (error != null) {
              return unlockCallback(error);
            }
            if (canProcess) {
              return callback(null, lock);
            }
            error = new Error('Refusing to process older message');
            error.code = 202;
            return unlockCallback(error);
          });
        };
      })(this));
    };

    RedisLocker.prototype._isFutureTimestamp = function(callback) {
      return this.redis.get("cache:timestamp:" + this.uuid, (function(_this) {
        return function(error, previousTimestamp) {
          var isFuture;
          if (error != null) {
            return callback(error);
          }
          try {
            if (previousTimestamp != null) {
              previousTimestamp = JSON.parse(previousTimestamp);
            }
          } catch (error1) {
            error = error1;
          }
          isFuture = true;
          if (previousTimestamp != null) {
            isFuture = _this.timestamp > previousTimestamp;
          }
          if (!isFuture) {
            return callback(null, false);
          }
          return _this.redis.set("cache:timestamp:" + _this.uuid, JSON.stringify(_this.timestamp), function(error) {
            if (error != null) {
              return callback(error);
            }
            return callback(null, true);
          });
        };
      })(this));
    };

    return RedisLocker;

  })();

  module.exports = RedisLocker;

}).call(this);

//# sourceMappingURL=redis-locker.js.map
